#include <Redis_Provider.h>

using namespace inaf::oasbo::Providers;
RedisProvider::RedisProvider() :
		RedisProvider("127.0.0.1", 6379, "AstriDAQ_key") {
}

RedisProvider::RedisProvider(std::string ip, int port, std::string key) :
		ip(ip), port(port) {
	setDest(key);
}

int RedisProvider::write(PacketLib::BasePacket &packet) {
	return write(packet, this->dest);
}

int RedisProvider::write(PacketLib::BasePacket &packet, std::string key) {
	if (!isOpen()) {
		if (context) {
			std::cout << "Redis Provider Error: " << context->errstr
					<< std::endl;
		} else {
			std::cout << "Redis Provider Error: Can't allocate Redis context"
					<< std::endl;
		}
		return -1;
	}

	uint size = packet.getHeaderSize() + packet.getPayloadSize()
			+ packet.getTailSize();
	redisReply *r = (redisReply*) redisCommand(context, "LPUSH %s %b",
			key.c_str(), (char*) packet.getBinaryPointer(), size);
	if (r == NULL) {
		// Error executing Redis command
		return -1;
	}
	// Check the type of the Redis reply
	if (r->type == REDIS_REPLY_ERROR) {
		// Error executing Redis command
		std::cout << r->str << std::endl;
		freeReplyObject(r);
		return -1;
	}
	freeReplyObject(r);
	return size;
}

int RedisProvider::open() {
	if (isOpen())
		close();
	context = redisConnect(ip.c_str(), port);
	if (context == nullptr || context->err) {
		if (context) {
			std::cout << "Redis Provider Error: " << context->errstr
					<< std::endl;
		} else {
			std::cout << "Redis Provider Error: Can't allocate Redis context"
					<< std::endl;
		}
		return -1;
	}
	// flush queue
	redisReply *r = (redisReply*) redisCommand(context, "DEL %s",
			this->dest.c_str());
	if (r == NULL) {
		std::cerr << "Redis Provider Error: error in flush queue with cmd: DEL "
				<< this->dest.c_str() << std::endl;
		return -1;
	}
	if (r->type == REDIS_REPLY_ERROR) {
		// Error executing Redis command
		std::cerr << "Redis Provider Error: error in flush queue with cmd: DEL "
				<< this->dest.c_str() << std::endl;
		freeReplyObject(r);
		return -1;
	}

	return 1;
}

int RedisProvider::close() {
	if (context != nullptr && !context->err) {
		redisFree(this->context);
		context = nullptr;
	}
	return 1;
}

bool RedisProvider::isOpen() {
	return (context != nullptr && !context->err);
}

RedisProvider::~RedisProvider() {
	close();
}

